@@ -1,30 +1,30 @@
-// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2019 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues.
 
-#ifndef _PRO_DATA_H_
-#define _PRO_DATA_H_
+#ifndef PRO_DATA_H
+#define PRO_DATA_H
 
 #include <string>
 #include <map>
 #include "GetDPConfig.h"
 #include "ListUtils.h"
 
-#define _0D      0
-#define _1D      1
-#define _2D      2
-#define _3D      3
-#define _ALL     4
-#define _AXI     5
+#define DIM_0D      0
+#define DIM_1D      1
+#define DIM_2D      2
+#define DIM_3D      3
+#define DIM_ALL     4
+#define DIM_AXI     5
 
 #define NBR_MAX_RES           2500
 #define NBR_MAX_POS           10
 
-#define _PRE  1 // pre-processing
-#define _CAL  2 // processing
-#define _POS  3 // post-processing
-#define _CST  5 // update constraint
+#define STATUS_PRE  1 // pre-processing
+#define STATUS_CAL  2 // processing
+#define STATUS_POS  3 // post-processing
+#define STATUS_CST  5 // update constraint
 
 #define TIME_STATIC   1
 #define TIME_THETA    2
@@ -1152,12 +1152,42 @@ struct Operation {
       char    *String;
     } SetGlobalSolverOptions;
     struct {
-      List_T  *FieldsToSkip;
+      List_T  *ViewTags;
     } BroadcastFields;
+    struct {
+      int from;
+      List_T  *Names;
+      List_T  *id;
+    } BroadcastVariables;
+    struct {
+      int to;
+      List_T  *Names;
+      List_T  *id;
+    } GatherVariables;
+    struct {
+      int from;
+      List_T  *Names;
+      List_T  *id;
+    } ScatterVariables;
+    struct {
+      int from;
+      List_T  *Names;
+      List_T  *id;
+    } CheckVariables;
+    struct {
+      List_T  *Names;
+    } ClearVariables;
+    struct {
+      List_T  *Names;
+    } ClearVectors;
     struct {
       int useList;
       char *from, *to;
     } Copy;
+    struct {
+      int alphaIndex, betaIndex;
+      char *v1, *v2, *v3; // v3=alpha*v1+beta*v2
+    } AddVector;
     struct {
       char *algorithm;
       char *currentPoint; // input and ouput
@@ -1326,6 +1356,12 @@ struct IterativeLoopSystem {
 #define OPERATION_OPTIMIZER_FINALIZE       109
 #define OPERATION_SETEXTRAPOLATIONORDER    110
 #define OPERATION_SETINCREMENTASSOLUTION   111
+#define OPERATION_ADDVECTOR                112
+#define OPERATION_CLEARVARIABLES           113
+#define OPERATION_CHECKVARIABLES           114
+#define OPERATION_CLEARVECTORS             115
+#define OPERATION_GATHERVARIABLES          116
+#define OPERATION_SCATTERVARIABLES         117
 
 /* ChangeOfState.Type */
 #define CHANGEOFSTATE_NOCHANGE              0
@@ -1524,7 +1560,6 @@ struct PostOpSolutions {
 #define FORMAT_ADAPT                   9
 #define FORMAT_GMSH                   10
 #define FORMAT_GMSH_PARSED            11
-#define FORMAT_MATLAB                 14
 #define FORMAT_GNUPLOT                15
 #define FORMAT_REGION_TABLE           16
 #define FORMAT_REGION_VALUE           17
@@ -1678,6 +1713,7 @@ struct Element {
 
 /* Element.Type */
 #define POINT            (1<<0)
+
 #define LINE             (1<<1)
 #define TRIANGLE         (1<<2)
 #define QUADRANGLE       (1<<3)
@@ -1685,6 +1721,7 @@ struct Element {
 #define HEXAHEDRON       (1<<5)
 #define PRISM            (1<<6)
 #define PYRAMID          (1<<7)
+
 #define LINE_2           (1<<8)
 #define TRIANGLE_2       (1<<9)
 #define QUADRANGLE_2     (1<<10)
@@ -1693,9 +1730,22 @@ struct Element {
 #define PRISM_2          (1<<13)
 #define PYRAMID_2        (1<<14)
 #define QUADRANGLE_2_8N  (1<<16)
-#define LINE_4           (1<<17)
-#define TRIANGLE_4       (1<<18)
-#define TETRAHEDRON_4    (1<<19)
+
+#define LINE_3           (1<<17)
+#define TRIANGLE_3       (1<<18)
+#define QUADRANGLE_3     (1<<19)
+#define TETRAHEDRON_3    (1<<20)
+#define HEXAHEDRON_3     (1<<21)
+#define PRISM_3          (1<<22)
+#define PYRAMID_3        (1<<23)
+
+#define LINE_4           (1<<24)
+#define TRIANGLE_4       (1<<25)
+#define QUADRANGLE_4     (1<<26)
+#define TETRAHEDRON_4    (1<<27)
+#define HEXAHEDRON_4     (1<<28)
+#define PRISM_4          (1<<29)
+#define PYRAMID_4        (1<<30)
 
 /* Adapt.Type */
 #define P1 1
