@@ -1,22 +1,23 @@
-// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2019 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues.
 
-#ifndef _PRO_DEFINES_H_
-#define _PRO_DEFINES_H_
+#ifndef PRO_DEFINES_H
+#define PRO_DEFINES_H
 
 #include "GetDPConfig.h"
 #include "ProDefine.h"
 
 #if defined(HAVE_KERNEL)
 #include "Gauss.h"
+#endif
+
 #include "F.h"
 #include "BF.h"
 #include "GF.h"
 #include "Cal_Value.h"
 extern struct CurrentData Current;
-#endif
 
 struct StringXDefine  Mesh_Format[] = {
   {"gmsh"  , FORMAT_GMSH}, {"Gmsh"  , FORMAT_GMSH},
@@ -113,20 +114,34 @@ struct StringXDefine  Integration_SubType[] = {
 struct StringXDefine  Element_Type[] = {
   {"Point"          , POINT},
   {"Line"           , LINE},
-  {"Triangle"       , TRIANGLE},
-  {"Quadrangle"     , QUADRANGLE},
-  {"Tetrahedron"    , TETRAHEDRON},
-  {"Tetrahedron2"   , TETRAHEDRON_2},
-  {"Hexahedron"     , HEXAHEDRON},
-  {"Prism"          , PRISM},
-  {"Pyramid"        , PYRAMID},
   {"Line2"          , LINE_2},
+  {"Line3"          , LINE_3},
+  {"Line4"          , LINE_4},
+  {"Triangle"       , TRIANGLE},
   {"Triangle2"      , TRIANGLE_2},
+  {"Triangle3"      , TRIANGLE_3},
+  {"Triangle4"      , TRIANGLE_4},
+  {"Quadrangle"     , QUADRANGLE},
   {"Quadrangle2"    , QUADRANGLE_2},
   {"Quadrangle2_8N" , QUADRANGLE_2_8N},
-  {"Line4"          , LINE_4},
-  {"Triangle4"      , TRIANGLE_4},
+  {"Quadrangle3"    , QUADRANGLE_3},
+  {"Quadrangle4"    , QUADRANGLE_4},
+  {"Tetrahedron"    , TETRAHEDRON},
+  {"Tetrahedron2"   , TETRAHEDRON_2},
+  {"Tetrahedron3"   , TETRAHEDRON_3},
   {"Tetrahedron4"   , TETRAHEDRON_4},
+  {"Hexahedron"     , HEXAHEDRON},
+  {"Hexahedron2"    , HEXAHEDRON_2},
+  {"Hexahedron3"    , HEXAHEDRON_3},
+  {"Hexahedron4"    , HEXAHEDRON_4},
+  {"Prism"          , PRISM},
+  {"Prism2"         , PRISM_2},
+  {"Prism3"         , PRISM_3},
+  {"Prism4"         , PRISM_4},
+  {"Pyramid"        , PYRAMID},
+  {"Pyramid2"       , PYRAMID_2},
+  {"Pyramid3"       , PYRAMID_3},
+  {"Pyramid4"       , PYRAMID_4},
   {NULL             , TRIANGLE}
  } ;
 
@@ -279,7 +294,6 @@ struct StringXDefine  NormOf_Type[] = {
 } ;
 
 struct StringXPointer  Current_Value[] = {
-#if defined(HAVE_KERNEL)
   {"Time"     , &Current.Time},       {"DTime"     , &Current.DTime},
   {"Theta"    , &Current.Theta},      {"TimeStep"  , &Current.TimeStep},
   {"Iter"      , &Current.Iteration},
@@ -315,7 +329,6 @@ struct StringXPointer  Current_Value[] = {
   {"KSPIterations", &Current.KSPIterations}, {"KSPIts", &Current.KSPIterations},
   {"KSPIteration", &Current.KSPIteration},
   {"KSPResidual", &Current.KSPResidual},
-#endif
   {NULL       , NULL}
 } ;
 
@@ -352,7 +365,6 @@ struct StringXDefine  PostSubOperation_Format[] = {
   {"Unv"                 , FORMAT_NXUNV},
   {"NX"                  , FORMAT_NXUNV},
   {"Gnuplot"             , FORMAT_GNUPLOT },
-  {"Matlab"              , FORMAT_MATLAB },
   {"Adaptation"          , FORMAT_ADAPT },
   {NULL                  , FORMAT_GMSH }
 } ;
@@ -400,17 +412,65 @@ struct DefineXFunction  FunctionForGauss[] = {
 #if defined(HAVE_KERNEL)
   {POINT          , (CAST)Gauss_Point},
   {LINE           , (CAST)Gauss_Line},
-  {TRIANGLE       , (CAST)Gauss_Triangle},
-  {QUADRANGLE     , (CAST)Gauss_Quadrangle},
-  {TETRAHEDRON    , (CAST)Gauss_Tetrahedron},
-  {TETRAHEDRON_2  , (CAST)Gauss_Tetrahedron},
-  {HEXAHEDRON     , (CAST)Gauss_Hexahedron},
-  {PRISM          , (CAST)Gauss_Prism},
-  {PYRAMID        , (CAST)Gauss_Pyramid},
   {LINE_2         , (CAST)Gauss_Line},
+  {LINE_3         , (CAST)Gauss_Line},
+  {LINE_4         , (CAST)Gauss_Line},
+  {TRIANGLE       , (CAST)Gauss_Triangle},
   {TRIANGLE_2     , (CAST)Gauss_Triangle},
+  {TRIANGLE_3     , (CAST)Gauss_Triangle},
+  {TRIANGLE_4     , (CAST)Gauss_Triangle},
+  {QUADRANGLE     , (CAST)Gauss_Quadrangle},
   {QUADRANGLE_2   , (CAST)Gauss_Quadrangle},
   {QUADRANGLE_2_8N, (CAST)Gauss_Quadrangle},
+  {QUADRANGLE_3   , (CAST)Gauss_Quadrangle},
+  {QUADRANGLE_4   , (CAST)Gauss_Quadrangle},
+  {TETRAHEDRON    , (CAST)Gauss_Tetrahedron},
+  {TETRAHEDRON_2  , (CAST)Gauss_Tetrahedron},
+  {TETRAHEDRON_3  , (CAST)Gauss_Tetrahedron},
+  {TETRAHEDRON_4  , (CAST)Gauss_Tetrahedron},
+  {HEXAHEDRON     , (CAST)Gauss_Hexahedron},
+  {HEXAHEDRON_2   , (CAST)Gauss_Hexahedron},
+  {HEXAHEDRON_3   , (CAST)Gauss_Hexahedron},
+  {HEXAHEDRON_4   , (CAST)Gauss_Hexahedron},
+  {PRISM          , (CAST)Gauss_Prism},
+  {PRISM_2        , (CAST)Gauss_Prism},
+  {PRISM_3        , (CAST)Gauss_Prism},
+  {PRISM_4        , (CAST)Gauss_Prism},
+  {PYRAMID        , (CAST)Gauss_Pyramid},
+  {PYRAMID_2      , (CAST)Gauss_Pyramid},
+  {PYRAMID_3      , (CAST)Gauss_Pyramid},
+  {PYRAMID_4      , (CAST)Gauss_Pyramid},
+#else
+  {POINT          , NULL},
+  {LINE           , NULL},
+  {LINE_2         , NULL},
+  {LINE_3         , NULL},
+  {LINE_4         , NULL},
+  {TRIANGLE       , NULL},
+  {TRIANGLE_2     , NULL},
+  {TRIANGLE_3     , NULL},
+  {TRIANGLE_4     , NULL},
+  {QUADRANGLE     , NULL},
+  {QUADRANGLE_2   , NULL},
+  {QUADRANGLE_2_8N, NULL},
+  {QUADRANGLE_3   , NULL},
+  {QUADRANGLE_4   , NULL},
+  {TETRAHEDRON    , NULL},
+  {TETRAHEDRON_2  , NULL},
+  {TETRAHEDRON_3  , NULL},
+  {TETRAHEDRON_4  , NULL},
+  {HEXAHEDRON     , NULL},
+  {HEXAHEDRON_2   , NULL},
+  {HEXAHEDRON_3   , NULL},
+  {HEXAHEDRON_4   , NULL},
+  {PRISM          , NULL},
+  {PRISM_2        , NULL},
+  {PRISM_3        , NULL},
+  {PRISM_4        , NULL},
+  {PYRAMID        , NULL},
+  {PYRAMID_2      , NULL},
+  {PYRAMID_3      , NULL},
+  {PYRAMID_4      , NULL},
 #endif
   {0              , 0}
 } ;
@@ -419,6 +479,9 @@ struct DefineXFunction  FunctionForSingularGauss[] = {
 #if defined(HAVE_KERNEL)
   {TRIANGLE       , (CAST)GaussSingularR_Triangle},
   {QUADRANGLE     , (CAST)GaussSingularR_Quadrangle},
+#else
+  {TRIANGLE       , NULL},
+  {QUADRANGLE     , NULL},
 #endif
   {0              , 0}
 } ;
@@ -431,22 +494,28 @@ struct DefineXFunction  FunctionForGaussLegendre[] = {
   {QUADRANGLE     , (CAST)GaussLegendre_Quadrangle},
   {TETRAHEDRON    , (CAST)GaussLegendre_Tetrahedron},
   {HEXAHEDRON     , (CAST)GaussLegendre_Hexahedron},
+#else
+  {POINT          , NULL},
+  {LINE           , NULL},
+  {TRIANGLE       , NULL},
+  {QUADRANGLE     , NULL},
+  {TETRAHEDRON    , NULL},
+  {HEXAHEDRON     , NULL},
 #endif
   {0              , 0}
 } ;
 
 #define POI  POINT
-#define LIN  LINE | LINE_2 | LINE_4
-#define TRI  TRIANGLE | TRIANGLE_2 | TRIANGLE_4
-#define QUA  QUADRANGLE | QUADRANGLE_2 | QUADRANGLE_2_8N
-#define TET  TETRAHEDRON | TETRAHEDRON_2 | TETRAHEDRON_4
-#define HEX  HEXAHEDRON | HEXAHEDRON_2
-#define PRI  PRISM | PRISM_2
-#define PYR  PYRAMID | PYRAMID_2
+#define LIN  LINE | LINE_2 | LINE_3 | LINE_4
+#define TRI  TRIANGLE | TRIANGLE_2 | TRIANGLE_3 | TRIANGLE_4
+#define QUA  QUADRANGLE | QUADRANGLE_2 | QUADRANGLE_2_8N | QUADRANGLE_3 | QUADRANGLE_4
+#define TET  TETRAHEDRON | TETRAHEDRON_2 | TETRAHEDRON_3 | TETRAHEDRON_4
+#define HEX  HEXAHEDRON | HEXAHEDRON_2 | HEXAHEDRON_3 | HEXAHEDRON_4
+#define PRI  PRISM | PRISM_2 | PRISM_3 | PRISM_4
+#define PYR  PYRAMID | PYRAMID_2 | PYRAMID_3 | PYRAMID_4
 #define ALL  POI|LIN|TRI|QUA|TET|HEX|PRI|PYR
 
 struct StringX3Function3Nbr  BF_Function[] = {
-#if defined(HAVE_KERNEL)
   // H^1 Basis Functions and their gradients
 
   {"BF_Node",    (CAST)BF_Node,
@@ -906,7 +975,6 @@ struct StringX3Function3Nbr  BF_Function[] = {
 
   {"BF_Global"  , (CAST)BF_Global   , (CAST)BF_dGlobal , (CAST)BF_dInvGlobal , 0. , ALL, 0 },
   {"BF_dGlobal" , (CAST)BF_dGlobal  , (CAST)BF_Zero    , (CAST)BF_Global, 0. , ALL, 0 },
-#endif
 
   {NULL , NULL , NULL , NULL , 0. , ALL, 0 }
 } ;
@@ -926,7 +994,6 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   /* #Par = -1 => free number of Parameters ; = -2 free even number */
   /* #Arg ... same */
 
-#if defined(HAVE_KERNEL)
   // F_Math
   {"Exp"               , (CAST)F_Exp              ,   0,   1 },
   {"Log"               , (CAST)F_Log              ,   0,   1 },
@@ -953,6 +1020,7 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"Max"               , (CAST)F_Max              ,   0,   2 },
   {"Jn"                , (CAST)F_Jn               ,   0,   2 },
   {"JnComplex"         , (CAST)F_JnComplex        ,   0,   2 },
+  {"KnComplex"         , (CAST)F_KnComplex        ,   0,   2 },
   {"Yn"                , (CAST)F_Yn               ,   0,   2 },
   {"dJn"               , (CAST)F_dJn              ,   0,   2 },
   {"dYn"               , (CAST)F_dYn              ,   0,   2 },
@@ -1114,6 +1182,7 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   //{"dbdh_Vinch"            , (CAST)F_dbdh_Vinch            ,  -1,   3 }, // NOT USED FOR NOW (26/06/2016)
   //{"Update_Jk"             , (CAST)F_Update_Jk             ,  -1,   6 },  //kj+++ // NOT USED FOR NOW (26/06/2016)
   //{"Update_Jk_sd"          , (CAST)F_Update_Jk_sd          ,  -1,   6 },  //kj+++ // NOT USED FOR NOW (26/06/2016)
+
   {"Update_Cell_K"         , (CAST)F_Update_Cell_K         ,  -1,   4 },  //kj+++
   {"b_Vinch_K"             , (CAST)F_b_Vinch_K             ,  -1,   -1 },  // 1+3*2=7 //kj+++
   {"hr_Vinch_K"            , (CAST)F_hr_Vinch_K            ,  -1,   -1 },  // 1+3*1=4 //kj+++
@@ -1122,6 +1191,15 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"dbdh_Vinch_K"          , (CAST)F_dbdh_Vinch_K          ,  -1,   -1 },  // parameter is dimension {2},{3}, 1+3*2=7 //kj+++
   {"dhdb_Vinch_K"          , (CAST)F_dhdb_Vinch_K          ,  -1,   -1 },  // parameter is dimension {2},{3}, 1+3*2=7 //kj+++
 
+  {"Cell_EB"          , (CAST)F_Cell_EB          ,  -1,   3  },  //kj+++
+  {"b_EB"             , (CAST)F_b_EB             ,  -1,   -1 },  // 1+3*2=7 //kj+++
+  {"hrev_EB"          , (CAST)F_hrev_EB          ,  -1,   -1 },  // 1+3*1=4 //kj+++
+  {"Jrev_EB"          , (CAST)F_Jrev_EB          ,  -1,   -1 },  // 1+3*1=4 //kj+++
+  {"h_EB"             , (CAST)F_h_EB             ,  -1,   -1 },  // parameter is dimension {2},{3}, 3+3*2=9 //kj+++
+  {"dbdh_EB"          , (CAST)F_dbdh_EB          ,  -1,   -1 },  // parameter is dimension {2},{3}, 1+3*2=7 //kj+++
+  {"dhdb_EB"          , (CAST)F_dhdb_EB          ,  -1,   -1 },  // parameter is dimension {2},{3}, 1+3*2=7 //kj+++
+
+
   // F_MultiHar
   {"MHToTime"          , (CAST)F_MHToTime         ,   0,   2 },
 
@@ -1236,21 +1314,19 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"GradHelmholtz"     , (CAST)GF_GradHelmholtz    ,   2,   0 },
   {"NPxGradHelmholtz"  , (CAST)GF_NPxGradHelmholtz ,   2,   0 },
   {"NSxGradHelmholtz"  , (CAST)GF_NSxGradHelmholtz ,   2,   0 },
-#endif
 
   {NULL                , NULL                      ,   0,   0 }
 
 } ;
 
 struct FunctionXFunction  GF_Function[] = {
-#if defined(HAVE_KERNEL)
   {(CAST)GF_Laplace         , (CAST)GF_LaplacexForm        } ,
   {(CAST)GF_GradLaplace     , (CAST)GF_GradLaplacexForm    } ,
   {(CAST)GF_NPxGradLaplace  , (CAST)GF_NPxGradLaplacexForm } ,
   {(CAST)GF_NSxGradLaplace  , (CAST)GF_NSxGradLaplacexForm } ,
   {(CAST)GF_ApproximateLaplace  , (CAST)GF_ApproximateLaplacexForm  } ,
   {(CAST)GF_Helmholtz         , (CAST)GF_HelmholtzxForm        } ,
-#endif
+
   {NULL                       , NULL }
 } ;
 
