@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2019 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues.
 
 #include <sstream>
 #include <vector>
@@ -45,7 +45,7 @@ static void Info(int level, char *arg0)
   case 0 :
     fprintf(stderr,
 	    "GetDP, a General environment for the treatment of Discrete Problems\n"
-	    "Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege\n"
+	    "Copyright (C) 1997-2019 P. Dular and C. Geuzaine, University of Liege\n"
 	    "Usage: %s [file] [options]\n"
 	    "Processing options:\n"
 	    "  -pre 'Resolution'         pre-processing\n"
@@ -116,7 +116,7 @@ static void Info(int level, char *arg0)
 #endif
     fprintf(stderr, "Packaged by      : %s\n", GETDP_PACKAGER);
     fprintf(stderr, "Web site         : http://getdp.info\n");
-    fprintf(stderr, "Mailing list     : getdp@onelab.info\n");
+    fprintf(stderr, "Issue tracker    : https://gitlab.onelab.info/getdp/getdp/issues\n");
     break;
   }
   Message::Exit(0);
