@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2019 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues.
 //
 // Contributor(s):
 //   Jean-Francois Remacle
@@ -66,37 +66,38 @@ static int PointInRefElement (struct Element * Element, double u, double v, doub
   double ZERO = 1.e-6;
 
   switch(Element->Type) {
-  case LINE : case LINE_2 :
+  case LINE : case LINE_2 : case LINE_3 : case LINE_4 :
     if (u<-ONE || u>ONE){
       return(0);
     }
     return(1);
-  case TRIANGLE : case TRIANGLE_2 :
+  case TRIANGLE : case TRIANGLE_2 : case TRIANGLE_3 : case TRIANGLE_4 :
     if (u<-ZERO || v<-ZERO || u>(ONE-v)){
       return(0);
     }
     return(1);
   case QUADRANGLE : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
     if (u<-ONE  || v<-ONE || u>ONE || v>ONE){
       return (0);
     }
     return(1);
-  case TETRAHEDRON : case TETRAHEDRON_2 :
+  case TETRAHEDRON : case TETRAHEDRON_2 : case TETRAHEDRON_3 : case TETRAHEDRON_4 :
     if (u<-ZERO || v<-ZERO || w<-ZERO || u>(ONE-v-w)){
       return(0);
     }
     return(1);
-  case HEXAHEDRON : case HEXAHEDRON_2 :
+  case HEXAHEDRON : case HEXAHEDRON_2 : case HEXAHEDRON_3 : case HEXAHEDRON_4 :
     if (u<-ONE || v<-ONE || w<-ONE || u>ONE || v>ONE || w>ONE){
       return(0);
     }
     return(1);
-  case PRISM : case PRISM_2 :
+  case PRISM : case PRISM_2 : case PRISM_3 : case PRISM_4 :
     if (w>ONE || w<-ONE || u<-ZERO || v<-ZERO || u>(ONE-v)){
       return(0);
     }
     return(1);
-  case PYRAMID : case PYRAMID_2 :
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_3 : case PYRAMID_4 :
     if (u<(w-ONE) || u>(ONE-w) || v<(w-ONE) || v>(ONE-w) || w<-ZERO || w>ONE){
       return(0);
     }
@@ -280,16 +281,23 @@ static void Init_SearchGrid(struct Grid * Grid)
 	    Brick_P = (struct Brick*)List_Pointer(Grid->Bricks, index);
 	    switch(Element.GeoElement->Type){
 	    case LINE :        case LINE_2 :
+            case LINE_3 :      case LINE_4 :
 	      List_Add(Brick_P->p[0], &Element.GeoElement);
 	      break;
 	    case TRIANGLE :    case TRIANGLE_2 :
+            case TRIANGLE_3 :  case TRIANGLE_4 :
 	    case QUADRANGLE :  case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+            case QUADRANGLE_3 : case QUADRANGLE_4 :
 	      List_Add(Brick_P->p[1], &Element.GeoElement);
 	      break;
-	    case TETRAHEDRON : case TETRAHEDRON_2 :
-	    case HEXAHEDRON :  case HEXAHEDRON_2 :
-	    case PRISM :       case PRISM_2 :
-	    case PYRAMID :     case PYRAMID_2 :
+	    case TETRAHEDRON   : case TETRAHEDRON_2 :
+            case TETRAHEDRON_3 : case TETRAHEDRON_4 :
+	    case HEXAHEDRON    : case HEXAHEDRON_2 :
+            case HEXAHEDRON_3  : case HEXAHEDRON_4 :
+	    case PRISM         : case PRISM_2 :
+            case PRISM_3       : case PRISM_4 :
+	    case PYRAMID       : case PYRAMID_2 :
+            case PYRAMID_3     : case PYRAMID_4 :
 	      List_Add(Brick_P->p[2], &Element.GeoElement);
 	      break;
 	    }
@@ -372,9 +380,9 @@ void InWhichElement (struct Grid * Grid, List_T *ExcludeRegion_L,
      bounding box, and even more if we search for elements of
      dimension smaller than the current dimension. This way we can for
      example also find 1D elements with points not exactly on them. */
-  if ((Dim == _1D && Current.GeoData->Dimension == _3D) ||
-      (Dim == _1D && Current.GeoData->Dimension == _2D) ||
-      (Dim == _2D && Current.GeoData->Dimension == _3D))
+  if ((Dim == DIM_1D && Current.GeoData->Dimension == DIM_3D) ||
+      (Dim == DIM_1D && Current.GeoData->Dimension == DIM_2D) ||
+      (Dim == DIM_2D && Current.GeoData->Dimension == DIM_3D))
     tol = Current.GeoData->CharacteristicLength * 1.e-4; /* instead of 5.e-3 */
   else
     tol = Current.GeoData->CharacteristicLength * 1.e-8;
@@ -399,11 +407,11 @@ void InWhichElement (struct Grid * Grid, List_T *ExcludeRegion_L,
   }
 
   switch(Dim){
-  case _1D  : lowdim = 0 ; highdim = 0 ; break;
-  case _2D  : lowdim = 1 ; highdim = 1 ; break;
-  case _3D  : lowdim = 2 ; highdim = 2 ; break;
-  case _ALL :
-  default   : lowdim = 0 ; highdim = 2 ; break;
+  case DIM_1D  : lowdim = 0 ; highdim = 0 ; break;
+  case DIM_2D  : lowdim = 1 ; highdim = 1 ; break;
+  case DIM_3D  : lowdim = 2 ; highdim = 2 ; break;
+  case DIM_ALL :
+  default      : lowdim = 0 ; highdim = 2 ; break;
   }
 
   for(dim = highdim ; dim >= lowdim ; dim--) {
@@ -441,28 +449,33 @@ void xyz2uvwInAnElement (struct Element *Element,
   double   u_new, v_new, w_new;
   double   Error = 1.0 ;
   int      i, iter = 1 ;
-  int      ChainDim = _3D, Type_Dimension, Type_Jacobian ;
+  int      ChainDim = DIM_3D, Type_Dimension, Type_Jacobian ;
   double (*Get_Jacobian)(struct Element*, MATRIX3x3*) ;
 
   *u = *v = *w = 0.0;
 
-  if(Element->Type & (TETRAHEDRON|TETRAHEDRON_2|HEXAHEDRON|PRISM|PYRAMID))
-    ChainDim = _3D;
-  else if(Element->Type & (TRIANGLE|QUADRANGLE|TRIANGLE_2|QUADRANGLE_2))
-    ChainDim = _2D;
-  else if(Element->Type & (LINE|LINE_2))
-    ChainDim = _1D;
+  if(Element->Type & (TETRAHEDRON|TETRAHEDRON_2|TETRAHEDRON_3|TETRAHEDRON_4|
+                      HEXAHEDRON|HEXAHEDRON_2|HEXAHEDRON_3|HEXAHEDRON_4|
+                      PRISM|PRISM_2|PRISM_3|PRISM_4|
+                      PYRAMID|PYRAMID_2|PYRAMID_3|PYRAMID_4))
+    ChainDim = DIM_3D;
+  else if(Element->Type & (TRIANGLE|TRIANGLE_2|TRIANGLE_3|TRIANGLE_4|
+                           QUADRANGLE|QUADRANGLE_2|QUADRANGLE_2_8N|
+                           QUADRANGLE_3|QUADRANGLE_4))
+    ChainDim = DIM_2D;
+  else if(Element->Type & (LINE|LINE_2|LINE_3|LINE_4))
+    ChainDim = DIM_1D;
   else if(Element->Type & POINT)
-    ChainDim = _0D;
+    ChainDim = DIM_0D;
   else{
     Message::Error("Unknown type of element in xyz2uvwInAnElement");
     return;
   }
 
-  if (ChainDim == _1D && Current.GeoData->Dimension == _3D)
+  if (ChainDim == DIM_1D && Current.GeoData->Dimension == DIM_3D)
     Type_Jacobian = JACOBIAN_LIN;
-  else if((ChainDim == _1D && Current.GeoData->Dimension == _2D) ||
-	  (ChainDim == _2D && Current.GeoData->Dimension == _3D))
+  else if((ChainDim == DIM_1D && Current.GeoData->Dimension == DIM_2D) ||
+	  (ChainDim == DIM_2D && Current.GeoData->Dimension == DIM_3D))
     Type_Jacobian = JACOBIAN_SUR;
   else
     Type_Jacobian = JACOBIAN_VOL;
